#include "gui/gui.h"
#include "gui/designer.h"
using namespace VGUI;
#include "singleton.h"
#include "game.h"
extern GAME game;

Designer::Designer( Widget* w )
{
	filename = "";
	center = w->GetCenter();
	name = w->GetName();
	text = w->GetText();
	type = w->GetType();
	tip = w->GetTip();
	rel_width = w->GetRelWidth();
	rel_height = w->GetRelHeight();
	enabled = w->GetEnabled();
	selected = w->GetSelected();
	is_default = w->GetDefault();
	is_cancel = w->GetCancel();
	has_icon = w->GetHasIcon();
	width_auto = w->GetWidthAuto();
	height_auto = w->GetHeightAuto();
	font_size = w->GetFontSize();
	color_r = w->GetColorR();
	color_g = w->GetColorG();
	color_b = w->GetColorB();

        int _w, _h;
        string skin_path = game.settings.GetSkinPath();
	//TODO - replace these with proper textures
        bg_tex.Load( skin_path + "/textures/widgets/tab_page_background.png", false, _w, _h );
        cone_tex.Load( skin_path + "/textures/widgets/tog_off_down.png", false, _w, _h );
	cone_tex_half_w = _w / 3200.0 / 2.0;
	cone_tex_half_h = _h / 2400.0 / 2.0;
        road_tex.Load( skin_path + "/textures/widgets/joy_cal_box.png", false, _w, _h );
        start_arrow_tex.Load( skin_path + "/textures/widgets/wheel_up_r.png", false, _w, _h );

	//assume the length of the field is 100 meters
	scale = 100.0/rel_width;

	mode = DESIGNER_MODE_NONE;
	selected_cone = NULL;

	Load();
}

Designer::~Designer()
{
}

void Designer::Save()
{
	ofstream designerfile;
	string filename = game.settings.GetSettingsDir() + "/userdata/designer.lst";
	designerfile.open(filename.c_str());

	designerfile << game.autox_track.isOpenTrack() << endl;
	designerfile << game.autox_track.cones.size() << endl;

	std::vector<VERTEX*>::iterator i;
	for (i=game.autox_track.cones.begin();i<game.autox_track.cones.end();i++)
	{
		designerfile << (*i)->x << " "
			     << (*i)->y << " "
			     << (*i)->z << endl;
	}

	designerfile.close();
}

void Designer::Load()
{
	ifstream designerfile;
	string filename = game.settings.GetSettingsDir() + "/userdata/designer.lst";
	designerfile.open(filename.c_str());

	game.autox_track.cones.clear();

	if (designerfile.good())
	{
		unsigned int size, i;
		bool open_track_flag;
		designerfile >> open_track_flag;
		if (game.autox_track.isOpenTrack() != open_track_flag)
			game.autox_track.ToggleOpenClose();
		designerfile >> size;

		for (i = 0; i < size; i++)
		{
			VERTEX* p = new VERTEX();
			designerfile >> p->x;
			designerfile >> p->y;
			designerfile >> p->z;
			game.autox_track.cones.push_back(p);
		}

		designerfile.close();
	}
}

void Designer::Draw()
{
        Draw( 1.0 );
}

void Designer::Draw( float opacity )
{
	float center_x = center.GetXPercent();
	float center_y = center.GetYPercent();
	float rel_width_half = rel_width / 2.0f;
	float rel_height_half = rel_height / 2.0f;
	game.utility.Draw2D( center_x - rel_width_half, center_y - rel_height_half, center_x + rel_width_half, center_y + rel_height_half, &bg_tex, 0, opacity );

	float quad_x[4], quad_y[4];
	int index = 0;
	std::vector<VERTEX*>::iterator i;
	for (i=game.autox_track.cones.begin();i<game.autox_track.cones.end();i++)
	{
		quad_x[index] = (*i)->x * rel_width + center_x - rel_width_half;
		quad_y[index] = (*i)->z * rel_height + center_y - rel_height_half;
		float x1 = quad_x[index] - cone_tex_half_w;
		float y1 = quad_y[index] - cone_tex_half_h;
		float x2 = quad_x[index] + cone_tex_half_w;
		float y2 = quad_y[index] + cone_tex_half_h;
		game.utility.Draw2D( x1, y1, x2, y2, &cone_tex, 0, opacity );

		index++;
		if (index == 4)
		{
			//the vertex array is arranged in the following sequence -
			//left, right, left, right, left, right, ...
			game.utility.DrawQuad(quad_x[0],quad_y[0],quad_x[2],quad_y[2],quad_x[3],quad_y[3],quad_x[1],quad_y[1], &road_tex, opacity);
			quad_x[0] = quad_x[2];
			quad_y[0] = quad_y[2];
			quad_x[1] = quad_x[3];
			quad_y[1] = quad_y[3];
			index = 2;
		}
	}

	//close the track
	unsigned int no_of_cones = game.autox_track.cones.size();
	if ((!game.autox_track.isOpenTrack()) && (no_of_cones%2 == 0) && (no_of_cones > 1))
	{
		quad_x[0] = quad_x[2];
		quad_y[0] = quad_y[2];
		quad_x[1] = quad_x[3];
		quad_y[1] = quad_y[3];
		quad_x[2] = game.autox_track.cones[0]->x * rel_width + center_x - rel_width_half;
		quad_y[2] = game.autox_track.cones[0]->z *rel_height + center_y - rel_height_half;
		quad_x[3] = game.autox_track.cones[1]->x * rel_width + center_x - rel_width_half;
		quad_y[3] = game.autox_track.cones[1]->z * rel_height + center_y - rel_height_half;
		game.utility.DrawQuad(quad_x[0],quad_y[0],quad_x[2],quad_y[2],quad_x[3],quad_y[3],quad_x[1],quad_y[1], &road_tex, opacity);
	}

	//draw the starting arrow
	if (no_of_cones > 1)
	{
		quad_x[2] = game.autox_track.cones[0]->x * rel_width + center_x - rel_width_half;
		quad_y[2] = game.autox_track.cones[0]->z * rel_height + center_y - rel_height_half;
		quad_x[3] = game.autox_track.cones[1]->x * rel_width + center_x - rel_width_half;
		quad_y[3] = game.autox_track.cones[1]->z * rel_height + center_y - rel_height_half;
		DrawStartArrow(quad_x[2], quad_y[2], quad_x[3], quad_y[3]);
	}	

	//draw mode string
	string mode_string;
	if (mode == DESIGNER_MODE_PLACE)
	{
		mode_string = "Place Cone";
	}
	else if (mode == DESIGNER_MODE_MOVE)
	{
		mode_string = "Move Cone";
	}
	else
		return;

	game.font.Print(0.83, 0.92, mode_string.c_str(), 1, 6, 1, 1, 0, opacity);
}

bool Designer::MouseOver( float x, float y )
{
        float min_x, min_y, max_x, max_y;
        min_x = center.GetXPercent() - ( rel_width / 2.0f );
        min_y = center.GetYPercent() - ( rel_height / 2.0f );
        max_x = center.GetXPercent() + ( rel_width / 2.0f );
        max_y = center.GetYPercent() + ( rel_height / 2.0f ) + 0.01;

        bool inside = ( x > min_x ) && ( x < max_x ) && ( y > min_y ) && ( y < max_y );

	if (inside && mode == DESIGNER_MODE_MOVE && selected_cone)
	{
		float clicked_x, clicked_y;

		clicked_x = x - min_x;
		clicked_y = y - min_y;

		selected_cone->x = clicked_x / rel_width;
		selected_cone->y = 0;
		selected_cone->z = clicked_y / rel_height;
	}	

	return inside;
}

string Designer::MousePress( float x, float y )
{
	float clicked_x, clicked_y;

	clicked_x = x - (center.GetXPercent() - rel_width / 2.0f);
	clicked_y = y - (center.GetYPercent() - rel_height / 2.0f);

	VERTEX* p = new VERTEX();
	p->x = clicked_x / rel_width;
	p->y = 0;
	p->z = clicked_y / rel_height;

	if (mode == DESIGNER_MODE_PLACE)
		game.autox_track.cones.push_back(p);
	else if (mode == DESIGNER_MODE_MOVE)
	{
		std::vector<VERTEX*>::iterator i;
		for (i=game.autox_track.cones.begin();i<game.autox_track.cones.end();i++)
		{
			VERTEX diff = *p - *(*i);
			if (diff.len() < cone_tex_half_w * 2.0 / rel_width) selected_cone = *i;
		}
		delete p; //since it's not added to the list of cones
	}

	return "";
}

string Designer::MouseRelease( float x, float y )
{
	selected_cone = NULL;
	return "";
}

void Designer::Delete()
{
	if (game.autox_track.cones.size() == 0) return;

	delete game.autox_track.cones.back();
	game.autox_track.cones.pop_back();

	if ((game.autox_track.cones.size() % 2) != 0)
	{
		delete game.autox_track.cones.back();
		game.autox_track.cones.pop_back();
	}

	return;
}

void Designer::DrawStartArrow(float x0, float y0, float x1, float y1)
{
	float t_x = x0 - x1;
	float t_y = y0 - y1;

	float new_x = -t_y / 2.0;
	float new_y = t_x / 2.0;

	float new_x0 = x0 - t_x / 3.0;
	float new_y0 = y0 - t_y / 3.0;
	float new_x1 = x1 + t_x / 3.0;
	float new_y1 = y1 + t_y / 3.0;

	float x2 = new_x0 - new_x;
	float y2 = new_y0 - new_y;
	float x3 = new_x1 - new_x;
	float y3 = new_y1 - new_y;

	game.utility.DrawQuad(x2,y2,new_x0,new_y0,new_x1,new_y1,x3,y3, &start_arrow_tex, 1.0);
}	
