#include "physics.h"
#include "common.h" //used for list iteration macros
#include "game.h"
extern GAME game;

void PHYSICS::Init()
{
	if (id != NULL || collisionconfig != NULL || collisiondispatcher != NULL || collisionbroadphase != NULL)
		DeInit();
	
	collisionconfig = new btDefaultCollisionConfiguration();
	collisiondispatcher = new btCollisionDispatcher(collisionconfig);
	
	btVector3 worldAabbMin(-10000,-10000,-10000);
	btVector3 worldAabbMax(10000,10000,10000);
	
	collisionbroadphase = new btAxisSweep3(worldAabbMin,worldAabbMax);
	
	id = new btCollisionWorld(collisiondispatcher,collisionbroadphase,collisionconfig);
}

void PHYSICS::DeInit()
{
	objects.clear();
	
	if (id != NULL)
		delete id;
	id = NULL;
	
	if (collisionbroadphase != NULL)
		delete collisionbroadphase;
	collisionbroadphase = NULL;
	
	if (collisiondispatcher != NULL)
		delete collisiondispatcher;
	collisiondispatcher = NULL;
	
	if (collisionconfig != NULL)
		delete collisionconfig;
	collisionconfig = NULL;
}

void PHYSICS::Tick(const double dt)
{
	/*if (id != NULL)
		dWorldStep(id, dt);
	else
	{
		throw EXCEPTION(__FILE__, __LINE__, "Ticked physics before init");
	}*/
}

void PHYSICSOBJECT::InitTrimesh(const void * vertices,
                            int vstride, int vcount,
			    const void * faces, int fcount,
                            int istride, const void * normals, const PHYSICSOBJECTSETTINGS & objsettings)
{
	if (loaded)
		throw EXCEPTION(__FILE__, __LINE__, "Tried to double load a physics object");
	
	if (!worldid || !colspeedup)
		throw EXCEPTION(__FILE__, __LINE__, "Tried to init trimesh of a physics object not connected to a world");
	
	if (trimesh != NULL || trimesh_varray != NULL)
		DeInit();
	
	if (fcount % 3 != 0)
		throw EXCEPTION(__FILE__, __LINE__, "Face count is not a multiple of 3");
	
	trimesh_varray = new btTriangleIndexVertexArray(fcount/3, (int *)faces, istride, vcount, (float *) vertices, vstride);
	
	trimesh = new btBvhTriangleMeshShape(trimesh_varray, true);
	
	id.getWorldTransform().setIdentity();
	//id.setCollisionFlags(btCollisionObject::CF_STATIC_OBJECT);
	
	id.setCollisionShape(trimesh);
	
	//store object in the quadtree
	/*{
		btVector3 AabbMin,AabbMax;
		id.getCollisionShape()->getAabb(id.getWorldTransform(),AabbMin,AabbMax);
		VERTEX bboxmin(AabbMin.x(),AabbMin.y(),AabbMin.z());
		VERTEX bboxmax(AabbMax.x(),AabbMax.y(),AabbMax.z());
		bbox.SetFromCorners(bboxmin, bboxmax);
		float xsize = ((*qtbound2).x-(*qtbound1).x) / qtlevels;
		float zsize = ((*qtbound2).z-(*qtbound1).z) / qtlevels;
		float curx = (*qtbound1).x;
		float curz = (*qtbound1).z;
		for (int x = 0; x < qtlevels; x++,curx += xsize)
		{
			for (int z = 0; z < qtlevels; z++,curz += zsize)
			{
				AABB thisblock;
				thisblock.SetFromCorners(VERTEX(curx,(*qtbound1).y,curz),
						VERTEX(curx+xsize,(*qtbound2).y,curz+zsize));
				
				if (bbox.IntersectAABB(thisblock))
				{
					(*quadtree)[x][z].push_back(this);
				}
			}
		}
	}*/
	
	btVector3 AabbMin,AabbMax;
	id.getCollisionShape()->getAabb(id.getWorldTransform(),AabbMin,AabbMax);
	VERTEX bboxmin(AabbMin.x(),AabbMin.y(),AabbMin.z());
	VERTEX bboxmax(AabbMax.x(),AabbMax.y(),AabbMax.z());
	bbox.SetFromCorners(bboxmin, bboxmax);
	
	colspeedup->Add(this);
	
	short int group = 1;
	short int mask = 1;
	
	if (objsettings.IsMapObject())
	{
		group = 2;
		mask = 1;
	}
	else if (objsettings.IsPlayerObject())
	{
		group = 4;
		mask = 1;
	}
	else if (objsettings.IsRoadObject())
	{
		group = 8;
		mask = 1;
	}
	
	//group = 255;
	//mask = 255;
	
	id.setUserPointer(const_cast<void *>(objsettings.ObjID()));
	//id.setUserPointer(this);
	//cout << "set physics object user pointer to: " << id.getUserPointer() << endl;
	//cout << "set physics object user pointer to: " << settings.ObjID() << endl;
	
	worldid->addCollisionObject(&id, group, mask);
	
	settings = objsettings;
	
	loaded = true;
}

void PHYSICSOBJECT::InitTrimesh(VERTEXARRAY & varray, const PHYSICSOBJECTSETTINGS & objsettings)
{
	const float * vertices;
	int vcount;
	const int * faces;
	int fcount;
	float * normals(NULL);
	varray.GetVertices(vertices, vcount);
	varray.GetFaces(faces, fcount);
	InitTrimesh(vertices, sizeof(float)*3, vcount, faces, fcount, sizeof(int)*3, normals, objsettings);
}

void PHYSICSOBJECT::DeInit()
{
	if (loaded || worldid)
	{
		worldid->removeCollisionObject(&id);
	}
	
	if (trimesh != NULL)
		delete trimesh;
	trimesh = NULL;
	
	if (trimesh_varray != NULL)
		delete trimesh_varray;
	trimesh_varray = NULL;
}

PHYSICSOBJECT & PHYSICS::NewPhysicsObject()
{
	{
		PHYSICSOBJECT newobj;
		objects.push_back(newobj);
		if (id == NULL)
		{
			throw EXCEPTION(__FILE__, __LINE__, "Tried to add objects before physics was initialized");
		}
		else
		{
			//id->addCollisionObject(&newobj.GetBulletObject());
			objects.back().SetBulletWorld(id);
			objects.back().SetCollisionSpeedup(&colspeedup);
		}
	}
	return objects.back();
}

bool PHYSICS::PassesFilter(const PHYSICSCOLLISIONSETTINGS & settings, void * checkme) const
{
	bool exception = false;
	
	//int count = 1;
	ITERCONST(void *, settings.GetExceptionObjectIDs(), i)
	{
		//cout << count << ". " << *i << " =? " << checkme;
		exception = exception || (*i == checkme);
		
		//if (exception)
		//	cout << " (exception)";
		//cout << endl;
		//++count;
	}
	
	return !exception;
}

void PHYSICS::CollideRay(const VERTEX & position, const VERTEX & direction, const float length, list <PHYSICSCONTACT> & outputcontactlist, const PHYSICSCOLLISIONSETTINGS & settings)
{
	outputcontactlist.clear();
	
	//return;
	
	if (id == NULL)
		throw EXCEPTION(__FILE__, __LINE__, "Tried to collide objects before physics was initialized");
	
	VERTEX rayend = position + direction.ScaleR(length);
	
	btVector3 rayFrom(position.x, position.y, position.z);
	btVector3 rayTo (rayend.x, rayend.y, rayend.z);
	
	MultipleRayResultCallback colresult(rayFrom, rayTo);
	
	short int raymask = 0;
	if (settings.GetMapCollide())
		raymask |= 2;
	if (settings.GetPlayerCollide())
		raymask |= 4;
	if (settings.GetRoadCollide())
		raymask |= 8;
	
	//raymask = 255;
	
	list <PHYSICSOBJECT *> candidates;
	AABB raybox;
	if (settings.GetMapCollide())
	{
		VERTEX rstart(game.objects.GetSceneNode().TransformIntoLocalSpace(position));
		VERTEX rend(game.objects.GetSceneNode().TransformIntoLocalSpace(rayend));
		/*VERTEX rstart	(game.objects.GetSceneNode().TransformIntoWorldSpace(position));
		VERTEX rend	(game.objects.GetSceneNode().TransformIntoWorldSpace(rayend));*/
		raybox.SetFromCorners(rstart, rend);
	}
	else
	{
		raybox.SetFromCorners(position,rayend);
		//cout << "test" << endl;
	}
	//colspeedup.Query(position, direction, length, candidates);
	colspeedup.Query(raybox, candidates);
	
	btTransform rayFromTrans,rayToTrans;
	rayFromTrans.setIdentity();
	rayFromTrans.setOrigin(rayFrom);
	rayToTrans.setIdentity();
	rayToTrans.setOrigin(rayTo);
	
	//cout << "collision candidates: " << candidates.size() << endl;
	int count(1);
	ITERATE(PHYSICSOBJECT *, candidates, i)
	{
		btCollisionObject * collisionObject = &((*i)->GetBulletObject());
		
		if (collisionObject->getBroadphaseHandle()->m_collisionFilterGroup & raymask)
		{
			id->rayTestSingle(rayFromTrans,rayToTrans,
				collisionObject,
				collisionObject->getCollisionShape(),
				collisionObject->getWorldTransform(),
				colresult);
		}
		
		count++;
	}
	
	//id->rayTest(rayFrom, rayTo, colresult, raymask);
	
	//cout << "Ray collisions: " << colresult.results.size() << endl;
	
	if (!colresult.results.empty())
	{
		//HandleCollision(settings, &colresult, outputcontactlist);
		
		ITERATE(btCollisionWorld::LocalRayResult, colresult.results, i)
		{
			//cout << "processing collision" << endl;
			
			if (!i->m_collisionObject)
			{
				throw EXCEPTION(__FILE__, __LINE__, "Collision hit isn't tied to an object");
			}
			
			if (PassesFilter(settings, i->m_collisionObject->getUserPointer()))
			{
				PHYSICSCONTACT newcont;
				outputcontactlist.push_back(newcont);
				
				VERTEX normal(i->m_hitNormalLocal.x(),i->m_hitNormalLocal.y(),i->m_hitNormalLocal.z());
				normal = normal.normalize();
				
				btVector3 hp;
				hp.setInterpolate3(colresult.m_rayFromWorld,colresult.m_rayToWorld,i->m_hitFraction);
				outputcontactlist.back().Set(VERTEX(hp.x(),hp.y(),hp.z()),
						normal,
						(colresult.m_rayToWorld - colresult.m_rayFromWorld).length() * i->m_hitFraction,
						(PHYSICSOBJECT*)i->m_collisionObject->getUserPointer(),
						(PHYSICSOBJECT*)i->m_collisionObject->getUserPointer());
				
				//outputcontactlist.pop_back();
				
				//cout << "PHYSICS::COLLIDERAY Collision hit: raylength=" << (colresult.m_rayToWorld - colresult.m_rayFromWorld).length() << ", hitfraction=" << i->m_hitFraction << ", depth=" << (colresult.m_rayToWorld - colresult.m_rayFromWorld).length() * i->m_hitFraction << ", hitpoint=";VERTEX(hp.x(),hp.y(),hp.z()).DebugPrint();
				//cout << "Collision hit: raylength=" << (colresult.m_rayToWorld - colresult.m_rayFromWorld).length() << ", hitfraction=" << i->m_hitFraction << ", depth=" << (colresult.m_rayToWorld - colresult.m_rayFromWorld).length() * i->m_hitFraction << ", normal=";normal.DebugPrint();
			}
			//else cout << "discarded collision due to filtering" << endl;
		}
	}
	
	/*PHYSICSCONTACT newcont;
	outputcontactlist.push_back(newcont);
	VERTEX normal(0,1,0);
	btVector3 hp;
	outputcontactlist.back().Set(VERTEX(position.x,5,position.z), normal, position.y, NULL, NULL);*/
}

void PHYSICS::CollideSphere(const VERTEX & position, const float size, list <PHYSICSCONTACT> & outputcontactlist, const PHYSICSCOLLISIONSETTINGS & settings)
{
	outputcontactlist.clear();
	
	if (id == NULL)
		throw EXCEPTION(__FILE__, __LINE__, "Tried to collide objects before physics was initialized");
	
	btCollisionObject object;
	object.getWorldTransform().setIdentity();
	object.getWorldTransform().setOrigin(btVector3(position.x,position.y,position.z));
	btSphereShape sphere(size);
	object.setCollisionShape(&sphere);
	
	short int mask = 0;
	if (settings.GetMapCollide())
		mask |= 2;
	if (settings.GetPlayerCollide())
		mask |= 4;
	if (settings.GetRoadCollide())
		mask |= 8;
	
	id->addCollisionObject(&object, 1, mask);
	
	//do the actual collision query
	id->performDiscreteCollisionDetection();
	
	//find the number of hits
	int num_manifolds = id->getDispatcher()->getNumManifolds();
	
	for (int i=0; i < num_manifolds; i++)
	{
		//find the number of contacts
		btPersistentManifold* manifold = id->getDispatcher()->getManifoldByIndexInternal(i);
		btCollisionObject* obA = static_cast<btCollisionObject*>(manifold->getBody0());
		btCollisionObject* obB = static_cast<btCollisionObject*>(manifold->getBody1());
		manifold->refreshContactPoints(obA->getWorldTransform(),obB->getWorldTransform());
		
		btCollisionObject * otherobject(NULL);
		bool obAissphere(obA == &object);
		
		if (obAissphere)
			otherobject = obB;
		else
			otherobject = obA;
		
		int num_contacts = manifold->getNumContacts();
		for (int j=0; j < num_contacts; j++)
		{
			btManifoldPoint& pt = manifold->getContactPoint(j);
			btVector3 ptA = pt.getPositionWorldOnA();
			btVector3 ptB = pt.getPositionWorldOnB();
			VERTEX spherept, otherpt;
			
			if (obAissphere)
			{
				spherept.Set(ptA.x(), ptA.y(), ptA.z());
				otherpt.Set(ptB.x(), ptB.y(), ptB.z());
			}
			else
			{
				otherpt.Set(ptA.x(), ptA.y(), ptA.z());
				spherept.Set(ptB.x(), ptB.y(), ptB.z());
			}
			
			//cout << i+1 << ", " << j+1 << ": collision ";pa.DebugPrint();
			//pb.DebugPrint();
			//cout << i+1 << ", " << j+1 << ": depth: " << (pb - pa).len() << endl;
			
			if (PassesFilter(settings, otherobject->getUserPointer()))
			{
				PHYSICSCONTACT newcont;
				outputcontactlist.push_back(newcont);
				
				outputcontactlist.back().Set(otherpt,
					VERTEX(pt.m_normalWorldOnB.x(),pt.m_normalWorldOnB.y(),pt.m_normalWorldOnB.z()),
					(otherpt-spherept).len(),
					(PHYSICSOBJECT*)otherobject->getUserPointer(),
					(PHYSICSOBJECT*)otherobject->getUserPointer());
			}
		}
	}
	
	//remove the sphere from the world
	id->removeCollisionObject(&object);
}

void PHYSICS::CollideBox(const VERTEX & position, const QUATERNION & orientation, const VERTEX & dimensions, list <PHYSICSCONTACT> & outputcontactlist, const PHYSICSCOLLISIONSETTINGS & settings)
{
	outputcontactlist.clear();
	
	//return;
	
	if (id == NULL)
		throw EXCEPTION(__FILE__, __LINE__, "Tried to collide objects before physics was initialized");
	
	btCollisionObject object;
	object.getWorldTransform().setIdentity();
	object.getWorldTransform().setOrigin(btVector3(position.x,position.y,position.z));
	object.getWorldTransform().setRotation(btQuaternion(orientation.x,orientation.y,orientation.z,orientation.w));
	btBoxShape box(btVector3(dimensions.x*0.5,dimensions.y*0.5,dimensions.z*0.5));
	object.setCollisionShape(&box);
	
	short int mask = 0;
	if (settings.GetMapCollide())
		mask |= 2;
	if (settings.GetPlayerCollide())
		mask |= 4;
	if (settings.GetRoadCollide())
		mask |= 8;
	
	//id->addCollisionObject(&object, 1, mask);
	
	//determine collision candidates in a speedy way
	list <PHYSICSOBJECT *> candidates;
	AABB querybox;
	VERTEX boxpos1(position-dimensions.ScaleR(0.5));
	VERTEX boxpos2(position+dimensions.ScaleR(0.5));
	if (settings.GetMapCollide())
	{
		VERTEX rstart(game.objects.GetSceneNode().TransformIntoLocalSpace(boxpos1));
		VERTEX rend(game.objects.GetSceneNode().TransformIntoLocalSpace(boxpos2));
		querybox.SetFromCorners(rstart, rend);
	}
	else
	{
		querybox.SetFromCorners(boxpos1,boxpos2);
	}
	colspeedup.Query(querybox, candidates);
	
	ITERATE(PHYSICSOBJECT *, candidates, i)
	{
		btCollisionAlgorithm * algo;
		algo = id->getDispatcher()->findAlgorithm(&object, &((*i)->GetBulletObject()));
		if (!algo)
		{
			throw EXCEPTION(__FILE__, __LINE__, "Couldn't find collision algorithm");
		}
		btManifoldResult manifoldout(&object, &((*i)->GetBulletObject()));
		algo->processCollision(&object, &((*i)->GetBulletObject()), id->getDispatchInfo(), &manifoldout);
	}
	
	/*//add cadidates to a list of collision pairs
	btOverlappingPairCache pairs;
	ITERATE(PHYSICSOBJECT *, candidates, i)
	{
		pairs.addOverlappingPair(object.getBroadphaseHandle(),(*i)->GetBulletObject().getBroadphaseHandle());
	}
	
	//do the actual collision query
	//id->performDiscreteCollisionDetection();
	id->getDispatcher()->dispatchAllCollisionPairs(&pairs,id->getDispatchInfo(),id->getDispatcher());*/
	
	//find the number of hits
	int num_manifolds = id->getDispatcher()->getNumManifolds();
	
	//if (num_manifolds > 0) cout << num_manifolds << " manifolds" << endl;
	
	for (int i=0; i < num_manifolds; i++)
	{
		//find the number of contacts
		btPersistentManifold* manifold = id->getDispatcher()->getManifoldByIndexInternal(i);
		btCollisionObject* obA = static_cast<btCollisionObject*>(manifold->getBody0());
		btCollisionObject* obB = static_cast<btCollisionObject*>(manifold->getBody1());
		manifold->refreshContactPoints(obA->getWorldTransform(),obB->getWorldTransform());
		
		btCollisionObject * otherobject(NULL);
		bool obAisbox(obA == &object);
		
		if (obAisbox)
			otherobject = obB;
		else
			otherobject = obA;
		
		int num_contacts = manifold->getNumContacts();
		//if (num_contacts > 0) cout << i << ". " << num_contacts << " contacts" << endl;
		for (int j=0; j < num_contacts; j++)
		{
			btManifoldPoint& pt = manifold->getContactPoint(j);
			btVector3 ptA = pt.getPositionWorldOnA();
			btVector3 ptB = pt.getPositionWorldOnB();
			VERTEX boxpt, otherpt;
			
			if (obAisbox)
			{
				boxpt.Set(ptA.x(), ptA.y(), ptA.z());
				otherpt.Set(ptB.x(), ptB.y(), ptB.z());
			}
			else
			{
				otherpt.Set(ptA.x(), ptA.y(), ptA.z());
				boxpt.Set(ptB.x(), ptB.y(), ptB.z());
			}
			
			//cout << i+1 << ", " << j+1 << ": collision ";pa.DebugPrint();
			//pb.DebugPrint();
			//cout << i+1 << ", " << j+1 << ": depth: " << (pb - pa).len() << endl;
			
			if (PassesFilter(settings, otherobject->getUserPointer()))
			{
				PHYSICSCONTACT newcont;
				outputcontactlist.push_back(newcont);
				
				outputcontactlist.back().Set(otherpt,
					VERTEX(pt.m_normalWorldOnB.x(),pt.m_normalWorldOnB.y(),pt.m_normalWorldOnB.z()),
					(otherpt-boxpt).len(),
					(PHYSICSOBJECT*)otherobject->getUserPointer(),
					(PHYSICSOBJECT*)otherobject->getUserPointer());
			}
		}
		
		//manifold->clearManifold();
	}
	
	for (int i=0; i < num_manifolds; i++)
		id->getDispatcher()->releaseManifold(id->getDispatcher()->getManifoldByIndexInternal(0));
	
	//remove the temporary object from the world
	//id->removeCollisionObject(&object);
}

void PHYSICSOBJECT::SetPosition(const VERTEX & newpos)
{
	if (loaded)
	{
		id.getWorldTransform().setOrigin(btVector3(newpos.x,newpos.y,newpos.z));
	}
	else throw EXCEPTION(__FILE__, __LINE__, "Physics object not loaded yet");
}

void PHYSICSOBJECT::SetQuaternion(const QUATERNION & newquat)
{
	if (loaded)
		id.getWorldTransform().setRotation(btQuaternion(newquat.x,newquat.y,newquat.z,newquat.w));
	else throw EXCEPTION(__FILE__, __LINE__, "Physics object not loaded yet");
}

void PHYSICS::GenerateCollisionList(const PHYSICSCOLLISIONSETTINGS & settings, list <const PHYSICSOBJECT *> & collisionlist) const
{
	ITERCONST(PHYSICSOBJECT, objects, i)
	{
		if ((settings.GetMapCollide() && i->IsMapObject()) ||
			(settings.GetPlayerCollide() && i->IsPlayerObject()) ||
			(settings.GetRoadCollide() && i->IsRoadObject()))
		{
			bool foundexception = false;
			const list <void *> exceptions = settings.GetExceptionObjectIDs();
			ITERCONST(void *, exceptions, e)
			{
				if (i->ObjID() == *e)
					foundexception = true;
				//cout << i->GetSettings().ObjID() << "==" << *e << "?" << endl;
			}
			if (!foundexception)
				collisionlist.push_back(&(*i));
		}
	}
}

void PHYSICS::DeletePhysicsObject(PHYSICSOBJECT * objtodel)
{
	colspeedup.Delete(objtodel);
	
	list <PHYSICSOBJECT>::iterator todel = objects.end();
	ITERATE(PHYSICSOBJECT, objects, i)
	{
		if (&(*i) == objtodel)
		{
			todel = i;
		}
	}
	
	if (todel != objects.end())
		objects.erase(todel);
	else
		throw EXCEPTION(__FILE__, __LINE__, "Got asked to delete physics object that doesn't exist");
}

void COLLISION_NODE::Add(PHYSICSOBJECT * object)
{
	objects.push_back(object);
	if (objects.size() == 1) //don't combine if this is the first object, otherwise the AABB would be enforced to include (0,0,0)
		bbox.SetTo(object->GetBBOX());
	else
		bbox.CombineWith(object->GetBBOX());
}

void COLLISION_NODE::CollapseTo(COLLISION_NODE & collapse_target)
{
	//send all leaves to the target node, clearing out everything else
	if (this != &collapse_target)
	{
		ITERATE(PHYSICSOBJECT *, objects, i)
		{
			collapse_target.Add(*i);
		}
		objects.clear();
		
		ITERATE(COLLISION_NODE, children, i)
		{
			i->CollapseTo(collapse_target);
		}
		children.clear();
	}
}

void COLLISION_NODE::RemoveDuplicateObjects()
{
	objects.sort();
	objects.unique();
}

void COLLISION_NODE::DebugPrint(int level, int & objectcount)
{
	for (int i = 0; i < level; i++)
		cout << "-";
	
	cout << "objects: " << objects.size() << ", child nodes: " << children.size() << ", aabb: ";
	bbox.DebugPrint();
	
	objectcount += objects.size();
	
	ITERATE(COLLISION_NODE, children, i)
		i->DebugPrint(level+1, objectcount);
	
	if (level == 0)
	{
		cout << "================" << endl;
		cout << "TOTAL OBJECTS: " << objectcount << endl;
	}
}

void COLLISION_NODE::DistributeObjectsToChildren(int level)
{	
	const unsigned int ideal_objects_per_node(1);
	const unsigned int ideal_children_per_node(2);
	const bool verbose(false);
	
	//enforce the rules:  i don't want to start with any children,
	// so tell all children to send me their objects, then delete them
	ITERATE(COLLISION_NODE, children, i)
	{
		i->CollapseTo(*this);
	}
	children.clear();
	
	//only continue if we have more objects than we need
	if (objects.size() <= ideal_objects_per_node)
		return;
	
	children.resize(ideal_children_per_node);
	
	/*//set parentage
	ITERATE(COLLISION_NODE, children, i)
	{
		i->parent = this;
	}*/
	
	//determine the average center position of all objects
	VERTEX avgcenter;
	int numobj = objects.size();
	float incamount = 1.0 / numobj;
	ITERATE(PHYSICSOBJECT *,objects,i)
	{
		avgcenter = avgcenter + (*i)->GetBBOX().GetCenter().ScaleR(incamount);
	}
	
	//find axis of maximum change, so we know where to split
	VERTEX axismask(1,0,0);
	VERTEX bboxsize = bbox.GetSize();
	if (bboxsize.x > bboxsize.y && bboxsize.x > bboxsize.z)
	{
		axismask.Set(1,0,0);
	}
	else if (bboxsize.y > bboxsize.x && bboxsize.y > bboxsize.z)
	{
		axismask.Set(0,1,0);
	}
	else if (bboxsize.z > bboxsize.y && bboxsize.z > bboxsize.x)
	{
		axismask.Set(0,0,1);
	}
	
	//cout << level << endl;
	
	//distribute objects to each child
	float avgcentercoord = avgcenter.dot(axismask);
	int distributor(0);
	ITERATE(PHYSICSOBJECT *, objects, i)
	{
		float objcentercoord = (*i)->GetBBOX().GetCenter().dot(axismask);
		
		if (objcentercoord > avgcentercoord)
			children.front().Add(*i);
		else if (objcentercoord < avgcentercoord)
			children.back().Add(*i);
		else
		{
			//cout << "unusual case #" << distributor << endl;
			
			//distribute children that sit right on our average center in an even way
			if (distributor % 2 == 0)
				children.front().Add(*i);
			else
				children.back().Add(*i);
			distributor++;
		}
	}
	
	//we've given away all of our objects; clear them out
	objects.clear();
	
	//count objects that belong to our children
	int child1obj = children.front().objects.size();
	int child2obj = children.back().objects.size();
	
	if (verbose) cout << "Objects: " << objects.size() << ", Child nodes: " << children.size() << " L obj: " << child1obj << " R obj: " << child2obj << endl;
	
	//if one child doesn't have any objects, then delete both children and take back their objects
	if (child1obj == 0 || child2obj == 0)
	{
		ITERATE(COLLISION_NODE, children, i)
		{
			ITERATE(PHYSICSOBJECT *, i->objects, n)
			{
				Add(*n);
			}
		}
		
		children.clear();
	}
	
	ITERATE(COLLISION_NODE, children, i)
		i->DistributeObjectsToChildren(level + 1);
}

/*void COLLISION_NODE::Query(const VERTEX & orig, const VERTEX & d, float seglen, list <PHYSICSOBJECT *> &outputlist, list <const COLLISION_NODE *> & cachenode_list_output) const
{
	//if we've got objects, test them
	ITERCONST(PHYSICSOBJECT *, objects, i)
	{
		if ((*i)->GetBBOX().IntersectSegment(orig, d, seglen))
		{
			outputlist.push_back(*i);
			cachenode_list_output.push_front(this);
		}
	}
	
	//if we have children, test them
	ITERCONST(COLLISION_NODE, children, i)
	{
		if (i->GetBBOX().IntersectSegment(orig, d, seglen))
		{
			//our child intersects with the segment, dispatch a query
			i->Query(orig, d, seglen, outputlist, cachenode_list_output);
		}
	}
}*/

void COLLISION_NODE::Query(const AABB & querybox, list <PHYSICSOBJECT *> &outputlist) const
{
	//if we've got objects, test them
	ITERCONST(PHYSICSOBJECT *, objects, i)
	{
		if ((*i)->GetBBOX().IntersectAABB(querybox))
		{
			outputlist.push_back(*i);
		}
	}
	
	//if we have children, test them
	ITERCONST(COLLISION_NODE, children, i)
	{
		if (i->GetBBOX().IntersectAABB(querybox))
		{
			//our child intersects with the segment, dispatch a query
			i->Query(querybox, outputlist);
		}
	}
}

void COLLISION_NODE::Delete(PHYSICSOBJECT * object)
{
	list <list <PHYSICSOBJECT *>::iterator> todel;
	
	//if we've got objects, test them
	ITERATE(PHYSICSOBJECT *, objects, i)
	{
		if (*i == object)
		{
			todel.push_back(i);
		}
	}
	
	//do any deletions
	ITERATE(list <PHYSICSOBJECT *>::iterator, todel, i)
	{
		objects.erase(*i);
	}
	
	//if we have children, pass it on
	ITERATE(COLLISION_NODE, children, i)
	{
		if (i->GetBBOX().IntersectAABB(object->GetBBOX()))
			i->Delete(object);
	}
}

void PHYSICS::OptimizeObjects()
{
	colspeedup.Optimize();
}

/*void COLLISION_SPEEDUP::Query(const VERTEX & orig, const VERTEX & d, float seglen, list <PHYSICSOBJECT *> &outputlist)
{

		head.Query(orig, d, seglen, outputlist, cachenode_list);

}*/

void COLLISION_SPEEDUP::Query(const AABB & querybox, list <PHYSICSOBJECT *> &outputlist)
{
	/*
	list <const COLLISION_NODE *> junklist;
	ITERATE(const COLLISION_NODE *, cachenode_list, i)
	{
		(*i)->Query(querybox, outputlist, junklist);
	}
	
	if (outputlist.empty()) //this assumes that any contact at all means we can skip an exhaustive search
	{*/
		//head.Query(querybox, outputlist, cachenode_list);
	
	/*	cachenode_list.unique();
		if (cachenode_list.size() > 10)
			cachenode_list.resize(10);
	}*/
	
	int startx = ConvertToGridX(querybox.GetPos().x);
	int endx = ConvertToGridX(querybox.GetPos().x+querybox.GetSize().x)+1;
	int startz = ConvertToGridZ(querybox.GetPos().z);
	int endz = ConvertToGridZ(querybox.GetPos().z+querybox.GetSize().z)+1;
	
	for (int x = startx; x < endx; x++)
	{
		for (int z = startz; z < endz; z++)
		{
			if (collision_nodes[x][z])
			{
				collision_nodes[x][z]->Query(querybox, outputlist);
				//if (!outputlist.empty())
				//	cout << outputlist.size() << endl;
			}
		}
	}
}

void COLLISION_SPEEDUP::Add(PHYSICSOBJECT * object)
{
	AABB & bbox = object->GetBBOX();
	
	int startx = ConvertToGridX(bbox.GetPos().x);
	int endx = ConvertToGridX(bbox.GetPos().x+bbox.GetSize().x)+1;
	int startz = ConvertToGridZ(bbox.GetPos().z);
	int endz = ConvertToGridZ(bbox.GetPos().z+bbox.GetSize().z)+1;
	
	//cout << startx << "," << endx << "-" << startz << "," << endz << "  "; bbox.DebugPrint();
	for (int x = startx; x < endx; x++)
	{
		for (int z = startz; z < endz; z++)
		{
			if (!collision_nodes[x][z])
			{
				collision_nodes[x][z] = new COLLISION_NODE;
			}
			
			collision_nodes[x][z]->Add(object);
			
			//object_map[object].push_back(collision_nodes[x][z]);
			object_map[object].push_back(pair<int,int>(x,z));
		}
	}
}

unsigned int COLLISION_SPEEDUP::ConvertToGridX(float val)
{
	int gridpos = (int)(((val - totalsize.GetPos().x)*(float) GetDimensionX())/totalsize.GetSize().x);
	if (gridpos >= (int)GetDimensionX())
		gridpos = (int)GetDimensionX()-1;
	if (gridpos < 0) 
		gridpos = 0;
	return gridpos;
}

unsigned int COLLISION_SPEEDUP::ConvertToGridZ(float val)
{
	int gridpos = (int)(((val - totalsize.GetPos().z)*(float) GetDimensionZ())/totalsize.GetSize().z);
	if (gridpos >= (int) GetDimensionZ())
		gridpos = (int) GetDimensionZ()-1;
	if (gridpos < 0) 
		gridpos = 0;
	return gridpos;
}

void COLLISION_SPEEDUP::Delete(PHYSICSOBJECT * object)
{
	/*for (unsigned int x = 0; x < GetDimensionX(); x++) 
	{
		for (unsigned int y = 0; y < GetDimensionZ(); y++)
		{
			if (collision_nodes[x][y] != NULL)
			{
				collision_nodes[x][y]->Delete(object);
				if (collision_nodes[x][y]->Empty())
				{
					delete collision_nodes[x][y];
					collision_nodes[x][y] = NULL;
				}
			}
		}
	}*/
	
	list <pair <int,int> > & object_usage = object_map[object];
	for (list <pair <int,int> >::iterator i = object_usage.begin(); i != object_usage.end(); i++)
	//ITERATE((pair <int,int>), object_usage, i)
	{
		int x = i->first;
		int z = i->second;
		collision_nodes[x][z]->Delete(object);
		if (collision_nodes[x][z]->Empty())
		{
			delete collision_nodes[x][z];
			collision_nodes[x][z] = NULL;
		}
	}
}

COLLISION_SPEEDUP::COLLISION_SPEEDUP(int square_dimensions, float nsx, float nsz)
{
	collision_nodes.resize(square_dimensions);
	totalsize.SetFromCorners(VERTEX(-nsx,0,-nsz),VERTEX(nsx,0,nsz));
	for (int i = 0; i < square_dimensions; i++)
		collision_nodes[i].resize(square_dimensions, NULL);
}

COLLISION_SPEEDUP::~COLLISION_SPEEDUP()
{
	for (unsigned int x = 0; x < GetDimensionX(); x++)
	{
		for (unsigned int z = 0; z < GetDimensionZ(); z++)
		{
			if (collision_nodes[x][z])
				delete collision_nodes[x][z];
			collision_nodes[x][z] = NULL;
		}
	}
}

void COLLISION_SPEEDUP::Optimize()
{
	for (unsigned int x = 0; x < GetDimensionX(); x++) 
	{
		for (unsigned int y = 0; y < GetDimensionZ(); y++)
		{
			if (collision_nodes[x][y])
			{
				collision_nodes[x][y]->CollapseTo(*collision_nodes[x][y]);
					
				collision_nodes[x][y]->RemoveDuplicateObjects();
					
				collision_nodes[x][y]->DistributeObjectsToChildren(0);
				
				//int numobj;
				//collision_nodes[x][y]->DebugPrint(0,numobj);
			}
		}
	}
}

PHYSICS::PHYSICS() : id(NULL),collisionconfig(NULL),collisiondispatcher(NULL),collisionbroadphase(NULL),
	colspeedup(256,1000.0,1000.0)
{}
