#include "shader.h"
#include "exception.h"
#include "game.h"
extern GAME game;
#include "common.h"

void SHADER_MANAGER::Init()
{
}

void SHADER_MANAGER::DeInit()
{
	if (!shadermap.empty())
		glUseProgramObjectARB(0);
	shadermap.clear();
}

bool SHADER::Load(const string & name)
{
	string vertexshader_source, fragmentshader_source;
	LoadFileIntoString(game.settings.GetFullDataPath("shaders/"+name+"/vertex.glsl"), vertexshader_source);
	LoadFileIntoString(game.settings.GetFullDataPath("shaders/"+name+"/fragment.glsl"), fragmentshader_source);
	
	// Create Shader And Program Objects
	program = glCreateProgramObjectARB();
	vertex_shader = glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
	fragment_shader = glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
	
	// Load Shader Sources
	GLcharARB vertshad[vertexshader_source.length()+1];
	strcpy(vertshad, vertexshader_source.c_str());
	const GLcharARB * vertshad2 = vertshad;
	//glShaderSourceARB(vertex_shader, 1, (GLcharARB**)(&(vertexshader_source.c_str())), NULL);
	glShaderSourceARB(vertex_shader, 1, &vertshad2, NULL);
	//glShaderSourceARB(fragment_shader, 1, fragmentshader_source.c_str(), NULL);
	GLcharARB fragshad[fragmentshader_source.length()+1];
	strcpy(fragshad, fragmentshader_source.c_str());
	const GLcharARB * fragshad2 = fragshad;
	glShaderSourceARB(fragment_shader, 1, &fragshad2, NULL);
	
	// Compile The Shaders
	GLint vertex_compiled(0);
	GLint fragment_compiled(0);
	
	glCompileShaderARB(vertex_shader);
	glCompileShaderARB(fragment_shader);
	
	glGetObjectParameterivARB(vertex_shader,
		GL_OBJECT_COMPILE_STATUS_ARB, &vertex_compiled);
	glGetObjectParameterivARB(fragment_shader,
		GL_OBJECT_COMPILE_STATUS_ARB, &fragment_compiled);
	
	// Attach The Shader Objects To The Program Object
	glAttachObjectARB(program, vertex_shader);
	glAttachObjectARB(program, fragment_shader);
	
	// Link The Program Object
	glLinkProgramARB(program);
	
	shadername = name;

	//need to enable to be able to set passed variable info
	Enable();
	
	//set passed variable information for tus
	for (int i = 0; i < 16; i++)
	{
		stringstream tustring;
		tustring << "tu" << i;
		int tu_loc;
		tu_loc = glGetUniformLocationARB(program, (tustring.str()+"_2D").c_str());
		if (tu_loc >= 0) glUniform1iARB(tu_loc, i);
			
		tu_loc = glGetUniformLocationARB(program, (tustring.str()+"_2DRect").c_str());
		if (tu_loc >= 0) glUniform1iARB(tu_loc, i);
			
		tu_loc = glGetUniformLocationARB(program, (tustring.str()+"_cube").c_str());
		if (tu_loc >= 0)
		{
			glUniform1iARB(tu_loc, i);
			//cout << "Uploaded to " << tustring.str()+"_cube" << endl;
		}
		
		//cout << tustring.str() + "_cube" << ": (" << tu_loc << ") " << i << endl;
	}
		
	const bool success = (vertex_compiled && fragment_compiled);
	
	//spit out any debugging info
	unsigned int logsize = 65536;
	char shaderlog[logsize];
	GLsizei loglength;
	glGetInfoLogARB(program, logsize, &loglength, shaderlog);
	//if (loglength > 0 && !(loglength == 102 || loglength == 180 || loglength == 181))
	if (loglength > 0)
	{
		cout << loglength << endl;
		game.WriteDebuggingData("----- Start Shader Log for " + name + " -----");
		game.WriteDebuggingData(shaderlog);
		game.WriteDebuggingData("----- End Shader Log for " + name + " -----");
		//return false;
	}
	
	if (!success)
	{
		game.WriteDebuggingData("Shader compilation failure: " + name);
	}
	
	loaded = success;
	
	return success;
}

void SHADER_MANAGER::EndScene()
{
	ITERMAPNOCONST ( string, SHADER, shadermap, i )
	{
		i->second.EndScene();
	}

	/*if (activeshader != shadermap.end())
		activeshader->second.EndScene();
	else
	{
		throw EXCEPTION(__FILE__,__LINE__,"Beginning a scene without any active shader!");
	}*/

	if ( !shadermap.empty() )
		glUseProgramObjectARB ( 0 );
}

void SHADER::EndScene()
{
	Enable();


	//set passed variable information for matrices
	/*int mat_loc = glGetUniformLocationARB(program, "prev_modelview");
	      if (mat_loc >= 0) 
	              (GLEW_VERSION_2_0) ? glUniformMatrix4fv(mat_loc, 1, GL_FALSE, game.graphics.GetPrevModelView()) : glUniformMatrix4fvARB(mat_loc, 1, GL_FALSE, game.graphics.GetPrevModelView()) ; 

	mat_loc = glGetUniformLocationARB(program, "prev_modelviewproj");
	      if (mat_loc >= 0) 
	              (GLEW_VERSION_2_0) ? glUniformMatrix4fv(mat_loc, 1, GL_FALSE, game.graphics.GetPrevModelViewProj()) :  glUniformMatrix4fvARB(mat_loc, 1, GL_FALSE, game.graphics.GetPrevModelViewProj());*/

	int loc = glGetUniformLocationARB ( program, "blurfactor" );
	if ( loc >= 0 )
		( GLEW_VERSION_2_0 ) ? glUniform1f ( loc, game.graphics.GetTrueFPS() *0.015 ) : glUniform1fARB ( loc, game.graphics.GetTrueFPS() *0.015 );

	loc = glGetUniformLocationARB ( program, "blurfactorx" );
	if ( loc >= 0 )
		( GLEW_VERSION_2_0 ) ? glUniform1f ( loc, game.graphics.GetW() /640.0 ) : glUniform1fARB ( loc, game.graphics.GetW() /640.0 );

	loc = glGetUniformLocationARB ( program, "blurfactory" );
	if ( loc >= 0 )
		( GLEW_VERSION_2_0 ) ? glUniform1f ( loc, game.graphics.GetH() /480.0 ) : glUniform1fARB ( loc, game.graphics.GetH() /480.0 );

	loc = glGetUniformLocationARB ( program, "screenw" );
	if ( loc >= 0 )
		( GLEW_VERSION_2_0 ) ? glUniform1f ( loc, game.graphics.GetW() ) : glUniform1fARB ( loc, game.graphics.GetW() );

	loc = glGetUniformLocationARB ( program, "screenh" );
	if ( loc >= 0 )
		( GLEW_VERSION_2_0 ) ? glUniform1f ( loc, game.graphics.GetH() ) : glUniform1fARB ( loc, game.graphics.GetH() );

	const QUATERNION & lightpos ( game.graphics.GetLightPosition() );
	VERTEX front;
	front.x = 1;
	loc = glGetUniformLocationARB ( program, "lightposition" );
	front = lightpos.RotateVec ( front );
	if ( loc >= 0 )
		( GLEW_VERSION_2_0 ) ? glUniform3f ( loc, -front.x, front.y, -front.z ) : glUniform3fARB ( loc, -front.x, front.y, -front.z );

	/*VERTEX eyelightpos(game.cam.dir.RotateVec(front));
	loc = glGetUniformLocationARB(program, "eyelightposition");
	      if (loc >= 0) 
	              (GLEW_VERSION_2_0) ? glUniform3f(loc, eyelightpos.x, eyelightpos.y, eyelightpos.z) : glUniform3fARB(loc, eyelightpos.x, eyelightpos.y, eyelightpos.z);*/

	/*loc = glGetUniformLocationARB(program, "screenratio");
	      if (loc >= 0) 
	              (GLEW_VERSION_2_0) ? glUniform1f(loc, game.graphics.GetH()/game.graphics.GetW()) : glUniform1fARB(loc, game.graphics.GetH()/game.graphics.GetW()); 
		
	loc = glGetUniformLocationARB(program, "screenratio_over2");
	      if (loc >= 0) 
	              (GLEW_VERSION_2_0) ? glUniform1f(loc, (game.graphics.GetH()/game.graphics.GetW())*0.5) : glUniform1fARB(loc, (game.graphics.GetH()/game.graphics.GetW())*0.5);*/

	float samples = 8;
	loc = glGetUniformLocationARB ( program, "samples" );
	if ( loc >= 0 )
		( GLEW_VERSION_2_0 ) ? glUniform1f ( loc, samples ) : glUniform1fARB ( loc, samples );

	loc = glGetUniformLocationARB ( program, "sm1inv" );
	if ( loc >= 0 )
		( GLEW_VERSION_2_0 ) ? glUniform1f ( loc, 1.0/ ( samples-1.0 ) ) : glUniform1fARB ( loc, 1.0/ ( samples-1.0 ) );

	loc = glGetUniformLocationARB ( program, "w" );
	if ( loc >= 0 )
		( GLEW_VERSION_2_0 ) ? glUniform1f ( loc, 1.0/samples ) : glUniform1fARB ( loc, 1.0/samples );

	//cout << game.graphics.GetTrueFPS() << endl;
}

void SHADER::LoadFileIntoString(const string & filepath, string & filestring)
{
	ifstream f;
	f.open(filepath.c_str());
	if (f)
	{
		char c[1024];
		
		while (f.good())
		{
			f.get(c, 1024, 0);
			filestring = filestring + c;
		}
		
		f.close();
	}
	else
	{
		throw EXCEPTION(__FILE__, __LINE__, "Shader file not found: " + filepath);
	}
}

SHADER::~SHADER()
{
	if (loaded) glDeleteObjectARB(program);
}

void SHADER::Enable()
{
	glUseProgramObjectARB(program);
}

/*void SHADER_MANAGER::SendObjectMatrix(MATRIX4 & matrix)
{
	//TODO: update for new matrix4 class
	//shaderlist[SHADER_BLURPASS].UploadMat16("prev_modelviewproj", matrix.Get());
}*/

bool SHADER::UploadMat16(const string & varname, float * mat16)
{
	int mat_loc = glGetUniformLocationARB(program, varname.c_str());
	if (mat_loc >= 0) glUniformMatrix4fv(mat_loc, 1, GL_FALSE, mat16);
	return (mat_loc >= 0);
}

//returns true on successful upload
bool SHADER::UploadActiveShaderParameter1i(const string & param, int val)
{
	int loc = glGetUniformLocationARB(program, param.c_str());
	if (loc >= 0) glUniform1i(loc, val);
	return (loc >= 0);
}

//returns true on successful upload
bool SHADER::UploadActiveShaderParameter1f(const string & param, float val)
{
	int loc = glGetUniformLocationARB(program, param.c_str());
	if (loc >= 0) glUniform1f(loc, val);
	return (loc >= 0);
}


void SHADER_MANAGER::BeginScene()
{
	if (activeshader != shadermap.end())
		activeshader->second.Enable();
	else
	{
		throw EXCEPTION(__FILE__,__LINE__,"Beginning a scene without any active shader!");
	}
}

void SHADER_MANAGER::SetActiveShader(const string & name)
{
	activeshader = shadermap.find(name);
	activeshader->second.Enable();
	if (activeshader == shadermap.end())
	{
		throw EXCEPTION(__FILE__,__LINE__,"Couldn't activate shader "+name +" because it's not loaded");
	}
}

bool SHADER_MANAGER::Load(const string & name)
{
	pair <map < string, SHADER >::iterator, bool> result = shadermap.insert(pair < string, SHADER > (name, SHADER()));
	map < string, SHADER >::iterator i = result.first;
	//if the insertion resulted in a new object being created, set it up
	bool success = true;
	if (result.second)
	{
		success = i->second.Load(name);
		game.WriteDebuggingData("Loaded shader package "+name);
	}
	
	return success;
}

void SHADER_MANAGER::UploadActiveShaderParameter1i(const string & param, int val)
{
	if (activeshader != shadermap.end())
	{
		//activeshader->second.Enable();
		if (!activeshader->second.UploadActiveShaderParameter1i(param, val))
			throw EXCEPTION(__FILE__,__LINE__,"Error on parameter upload for \""+param+"\"");
	}
	else
	{
		throw EXCEPTION(__FILE__,__LINE__,"No active shader!");
	}
}

void SHADER_MANAGER::UploadActiveShaderParameter1f(const string & param, float val)
{
	if (activeshader != shadermap.end())
	{
		//activeshader->second.Enable();
		if (!activeshader->second.UploadActiveShaderParameter1f(param, val))
			throw EXCEPTION(__FILE__,__LINE__,"Error on parameter upload for \""+param+"\"");
	}
	else
	{
		throw EXCEPTION(__FILE__,__LINE__,"No active shader!");
	}
}

void SHADER_MANAGER::UploadActiveMat16(const string & varname, float * mat16)
{
	if (activeshader != shadermap.end())
	{
		//activeshader->second.Enable();
		if (!activeshader->second.UploadMat16(varname, mat16))
			throw EXCEPTION(__FILE__,__LINE__,"Error on parameter upload to shader \""+activeshader->first+"\" for \""+varname+"\"");
	}
	else
	{
		throw EXCEPTION(__FILE__,__LINE__,"No active shader!");
	}
}
